%% Script Documentation
%This collapse pressure script is used for obtaining the hydrostatic 
%collapse profile for gas vesicle (GV) samples. The script interfaces with
%the Alicat pressure controller and the Ocean Optics 
%STS-VIS microspectrometer to ramp up the hydrostatic pressure from 
%0 bar to 14 bar in discrete user-defined steps and measure the light 
%transmitted through the sample at each step. 
%This version of the script outputs two Excel files. 
%The first one provides the Absorbance/Optical Density (OD)at 500 nm for the GV 
%sample as a function of applied hydrostatic pressure. This output Excel file 
%also contains the OD normalized with respect to the starting OD (at 0 bar) 
%for the intact GV sample, as well as the final OD (at 14 bar) for the fully collapsed 
%GV sample (blank). The second Excel file stores the full spectrum (absorbance 
%at all wavelengths in the spectrum) of the collapsed GV sample as a reference. 
%Optional: The user can also save the measured absorbance for the GV sample 
%at all wavelengths in the spectrum based on personal preference.

%% Startup instructions
%Prior to running this script on the MATLAB command window, 
%please follow the instructions on lines 2-5
%Let the spectrophotometer warm up for 30 min
%Run the Alicat_startup script
%Run OceanOptics_startup_FullTrans with the cuvette with buffer
%Run OceanOptics_startup_NoTrans with the cuvette turned sideways
      
%% User-defined parameters
EqualizationTime = 7; % Set waiting time per step
pressureRange = 10; %Set pressure range between 10 and 14 bar
pressureStep = 0.2; %Set pressure step size to 0.2 bar
blankPressure = 14; %Pressure to use for blank measurement
runName = 'Destination_subfolder\Filename';
%Note: the index for 499.8964 nm on the SIS-VIS spectrometer is 354
%Note: the index for 600 nm from the STS-VIS calibration data sheet is 565 

%% Initializing parameters
numSteps = pressureRange/pressureStep + 1;
pressureVector = linspace(0,pressureRange,numSteps)';
%Define the wavelength vector for spectrometer
wavelengths = invoke(spectrometerObj, 'getWavelengths', spectrometerIndex, channelIndex);
%Grab spectral data to use as initial intensity
specFull = FullTransmission; %Comes from OceanOptics_startup_FullTrans
specDark = NoTransmission; %Comes from OceanOptics_startup_NoTrans
specInit = specFull - specDark;
%Initialize spectral data for spectrometer
specData = invoke(spectrometerObj, 'getSpectrum', spectrometerIndex);
%Initialize matrix to store spectral data for each point
specMatrix = zeros(length(specData),numSteps);

%% Pressure cycling
for i = 1:numSteps
    %Pressure setpoint change
    setPressure = pressureVector(i);
    setAlicatPressure(alicat, setPressure) 
    
    %Implementation of wait time
    tic
    time = toc;
    while time < EqualizationTime
        time = toc;
    end
    time = 0;
    
    specData = invoke(spectrometerObj, 'getSpectrum', spectrometerIndex);
    specMatrix(:,i) = specData-specDark;
    disp([pressureVector(i), -log10(specMatrix(354,i)./specInit(354))]);
  
end

%% Blank measurement
%Zero point pressure for complete collapse blank
setAlicatPressure(alicat,  blankPressure)

%Implementation of wait time
tic
time = toc;
while time < EqualizationTime*2
    time = toc;     
end
time = 0;

%Post-collapse blank
specBlankTemp = invoke(spectrometerObj, 'getSpectrum', spectrometerIndex);
specBlank = specBlankTemp-specDark;

%% Reset Alicat pressure controller to zero
setAlicatPressure(alicat,0)

%% Converting raw spectral data to absorbance 
%Now have spectral data for each pressure value (specMatrix) and initial
%spectral scan (specInit), need to convert to absorbance value

%Initialize absorbance matrix
absMatrix = zeros(size(specMatrix));
%Convert specmatrix into an absorbance value, one for each wavelength
for i = 1:length(specMatrix(1,:))
    absMatrix(:,i) = -log10(specMatrix(:,i)./specInit);
end

%Now have an absorbance matrix for all wavelengths in spectrum, this can
%also be exported to a seperate Excel file according to the needs and discretion
%of the user.

%Calculate absorbance blank for fully collapsed GV sample
absBlank = -log10(specBlank./specInit);

%Calculate normalized absorbance for whole sample
normAbs = zeros(size(absMatrix));

for i = 1:numSteps
    normAbs(:,i) = (absMatrix(:,i)-absBlank)./(absMatrix(:,1)-absBlank);
    
end

%% Plotting for wavelength of 499.8964 nm versus pressure vector
figure(1)
plot(pressureVector,absMatrix(354,:),'Linewidth',2)
figure(2)
plot(pressureVector,normAbs(354,:),'Linewidth',2)

%% Export relevant measurements to an output Excel file
Pressure_Bar = pressureVector;
Absorbance500 = absMatrix(354,:)';
NormAbs500 = normAbs(354,:)';

filename = strcat('Destination_folder_path\',runName,'.xlsx');
exportTable = table(Pressure_Bar,Absorbance500,NormAbs500);
writetable(exportTable,filename);
filename = strcat('Destination_folder_path\',runName,'_spectrumblank','.xlsx');
exportTable2 = table(wavelengths,specBlank,absBlank);
writetable(exportTable2,filename);

%% Signals end of measurement run
beep;
disp('DONE!')

